@@ -38,6 +38,11 @@
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 
+#ifdef MAC_INTEGRATION
+#  include <ige-mac-integration.h>
+#endif
+
+
 #define ACTION_GROUP_STATIC             "GladeStatic"
 #define ACTION_GROUP_PROJECT            "GladeProject"
 #define ACTION_GROUP_PROJECTS_LIST_MENU "GladeProjectsList"
@@ -504,32 +509,6 @@ create_recent_chooser_menu (GladeWindow *window, GtkRecentManager *manager)
 	return recent_menu;
 }
 
-static void
-window_screen_changed_cb (GtkWidget *widget,
-			  GdkScreen *old_screen,
-			  GladeWindow *window)
-{
-	GtkWidget *menu_item;
-	GdkScreen *screen;
-
-	screen = gtk_widget_get_screen (widget);	
-
-	window->priv->recent_manager = gtk_recent_manager_get_for_screen (screen);
-
-	gtk_menu_detach (GTK_MENU (window->priv->recent_menu));
-	g_object_unref (G_OBJECT (window->priv->recent_menu));
-	
-	window->priv->recent_menu = create_recent_chooser_menu (window, window->priv->recent_manager);
-
-	g_signal_connect (window->priv->recent_menu,
-			  "item-activated",
-			  G_CALLBACK (recent_chooser_item_activated_cb),
-			  window);
-			  
-	menu_item = gtk_ui_manager_get_widget (window->priv->ui, "/MenuBar/FileMenu/OpenRecent");
-	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item), window->priv->recent_menu);
-}
-
 static void
 activate_action (GtkToolButton *toolbutton,
 				      GladeWidgetAction *action) 
@@ -1181,10 +1160,14 @@ save (GladeWindow *window, GladeProject *project, const gchar *path)
 	{
 		/* Reset path so future saves will prompt the file chooser */
 		glade_project_reset_path (project);
-		glade_util_ui_message (GTK_WIDGET (window), GLADE_UI_ERROR, NULL, 
-				       _("Failed to save %s: %s"),
-				       display_path, error->message);
-		g_error_free (error);
+
+		if (error)
+		{
+			glade_util_ui_message (GTK_WIDGET (window), GLADE_UI_ERROR, NULL, 
+					       _("Failed to save %s: %s"),
+					       display_path, error->message);
+			g_error_free (error);
+		}
 		g_free (display_path);
 		return;
 	}
@@ -1821,8 +1804,8 @@ toggle_dock_cb (GtkAction *action, GladeWindow *window)
 		toplevel = gtk_widget_get_toplevel (dock->widget);
 
 		g_object_ref (dock->widget);
-		gtk_container_remove (GTK_CONTAINER 
-				      (GTK_BIN (toplevel)->child), dock->widget);
+		gtk_container_remove (GTK_CONTAINER (gtk_bin_get_child (GTK_BIN (toplevel))),
+				      dock->widget);
 
 		if (dock->first_child)
 			gtk_paned_pack1 (GTK_PANED (dock->paned), dock->widget, FALSE, FALSE);
@@ -1864,8 +1847,8 @@ toggle_dock_cb (GtkAction *action, GladeWindow *window)
 		g_signal_connect (G_OBJECT (toplevel), "configure-event",
 				  G_CALLBACK (on_dock_resized), dock);
 
-		if (!GTK_PANED (dock->paned)->child1 &&
-		    !GTK_PANED (dock->paned)->child2)
+		if (!gtk_paned_get_child1 (GTK_PANED (dock->paned)) &&
+		    !gtk_paned_get_child2 (GTK_PANED (dock->paned)))
 			gtk_widget_hide (dock->paned);
 
 
@@ -2201,7 +2184,11 @@ menu_item_selected_cb (GtkWidget *item, GladeWindow *window)
 	GtkAction *action;
 	gchar *tooltip;
 
-        action = gtk_widget_get_action (item);
+#if (GTK_MAJOR_VERSION == 2) && (GTK_MINOR_VERSION < 16)
+	action = gtk_widget_get_action (item);
+#else
+	action = gtk_activatable_get_related_action (GTK_ACTIVATABLE (item));
+#endif
 	g_object_get (G_OBJECT (action), "tooltip", &tooltip, NULL);
 
 	if (tooltip != NULL)
@@ -2324,17 +2311,14 @@ drag_data_received (GtkWidget *widget,
 			guint info, guint time, GladeWindow *window)
 {
 	gchar **uris, **str;
-	gchar *data;
+	const guchar *data;
 
 	if (info != TARGET_URI_LIST)
 		return;
 
-	/* On MS-Windows, it looks like `selection_data->data' is not NULL terminated. */
-	data = g_new (gchar, selection_data->length + 1);
-	memcpy (data, selection_data->data, selection_data->length);
-	data[selection_data->length] = 0;
+	data = gtk_selection_data_get_data (selection_data);
 
-	uris = g_uri_list_extract_uris (data);
+	uris = g_uri_list_extract_uris ((gchar *) data);
 
 	for (str = uris; *str; str++)
 	{
@@ -2382,10 +2366,8 @@ create_selector_tool_button (GtkToolbar *toolbar)
 	gtk_tool_button_set_icon_widget (GTK_TOOL_BUTTON (button), image);
 	gtk_tool_button_set_label (GTK_TOOL_BUTTON (button), _("Select"));
 	
-	gtk_tool_item_set_tooltip (GTK_TOOL_ITEM (button),
-				   toolbar->tooltips,
-				   _("Select widgets in the workspace"),
-				   NULL);
+	gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (button),
+				        _("Select widgets in the workspace"));
 	
 	gtk_widget_show (GTK_WIDGET (button));
 	gtk_widget_show (image);
@@ -2410,10 +2392,8 @@ create_drag_resize_tool_button (GtkToolbar *toolbar)
 	gtk_tool_button_set_icon_widget (GTK_TOOL_BUTTON (button), image);
 	gtk_tool_button_set_label (GTK_TOOL_BUTTON (button), _("Drag Resize"));
 	
-	gtk_tool_item_set_tooltip (GTK_TOOL_ITEM (button),
-				   toolbar->tooltips,
-				   _("Drag and resize widgets in the workspace"),
-				   NULL);
+	gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (button),
+				        _("Drag and resize widgets in the workspace"));
 	
 	gtk_widget_show (GTK_WIDGET (button));
 	gtk_widget_show (image);
@@ -2871,10 +2851,10 @@ glade_window_set_initial_size (GladeWindow *window, GKeyFile *config)
 }
 
 static void
-load_paned_position (GKeyFile *config, GtkWidget *pane, const gchar *name)
+load_paned_position (GKeyFile *config, GtkWidget *pane, const gchar *name, gint default_position)
 {
 	gtk_paned_set_position (GTK_PANED (pane),
-				g_key_file_get_integer (config, name, "position", NULL));
+				key_file_get_int (config, name, "position", default_position));
 }
 
 static void
@@ -2884,9 +2864,9 @@ glade_window_config_load (GladeWindow *window)
 	
 	glade_window_set_initial_size (window, config);
 	
-	load_paned_position (config, window->priv->center_pane, "center_pane");
-	load_paned_position (config, window->priv->left_pane, "left_pane");
-	load_paned_position (config, window->priv->right_pane, "right_pane");
+	load_paned_position (config, window->priv->center_pane, "center_pane", 400);
+	load_paned_position (config, window->priv->left_pane, "left_pane", 200);
+	load_paned_position (config, window->priv->right_pane, "right_pane", 220);
 }
 
 static void
@@ -2946,7 +2926,7 @@ glade_window_init (GladeWindow *window)
 	GtkWidget *dockitem;
 	GtkWidget *widget;
 	GtkWidget *sep;
-	GtkAction *action;
+	GtkAction *undo_action, *redo_action;
 	GtkAccelGroup *accel_group;	
 
 	window->priv = priv = GLADE_WINDOW_GET_PRIVATE (window);
@@ -2984,11 +2964,16 @@ glade_window_init (GladeWindow *window)
 	gtk_toolbar_insert (GTK_TOOLBAR (priv->toolbar), GTK_TOOL_ITEM (priv->undo), 4);
 	gtk_toolbar_insert (GTK_TOOLBAR (priv->toolbar), GTK_TOOL_ITEM (priv->redo), 5);
 
-	action = gtk_ui_manager_get_action (priv->ui, "/MenuBar/EditMenu/Undo");
-	gtk_action_connect_proxy (action, GTK_WIDGET (priv->undo));
-
-	action = gtk_ui_manager_get_action (priv->ui, "/MenuBar/EditMenu/Redo");
-	gtk_action_connect_proxy (action, GTK_WIDGET (priv->redo));
+	undo_action = gtk_ui_manager_get_action (priv->ui, "/MenuBar/EditMenu/Undo");
+	redo_action = gtk_ui_manager_get_action (priv->ui, "/MenuBar/EditMenu/Redo");
+	
+#if (GTK_MAJOR_VERSION == 2) && (GTK_MINOR_VERSION < 16)
+	gtk_action_connect_proxy (undo_action, GTK_WIDGET (priv->undo));
+	gtk_action_connect_proxy (redo_action, GTK_WIDGET (priv->redo));
+#else
+	gtk_activatable_set_related_action (GTK_ACTIVATABLE (priv->undo), undo_action);
+	gtk_activatable_set_related_action (GTK_ACTIVATABLE (priv->redo), redo_action);
+#endif
 	
 	/* main contents */
 	hpaned1 = gtk_hpaned_new ();
@@ -3024,7 +3009,6 @@ glade_window_init (GladeWindow *window)
 	gtk_paned_pack1 (GTK_PANED (hpaned2), palette, FALSE, FALSE);
 	setup_dock (&priv->docks[DOCK_PALETTE], palette, 200, 540, 
 		    _("Palette"), "palette", hpaned2, TRUE);
-	gtk_widget_set_size_request (palette, 200, 540);
 	gtk_widget_show (palette);
 
 	/* inspectors */
@@ -3070,7 +3054,7 @@ glade_window_init (GladeWindow *window)
 	gtk_widget_show (vbox);
 	
 	/* recent files */	
-	priv->recent_manager = gtk_recent_manager_get_for_screen (gtk_widget_get_screen (GTK_WIDGET (window)));
+	priv->recent_manager = gtk_recent_manager_get_default ();
 
 	priv->recent_menu = create_recent_chooser_menu (window, priv->recent_manager);
 
@@ -3137,10 +3121,6 @@ glade_window_init (GladeWindow *window)
 			  window);
 			  
 	/* GtkWindow events */
-	g_signal_connect (window, "screen-changed",
-			  G_CALLBACK (window_screen_changed_cb),
-			  window);
-			  
 	g_signal_connect (window, "window-state-event",
 			  G_CALLBACK (window_state_event_cb),
 			  window);
@@ -3169,6 +3149,19 @@ glade_window_init (GladeWindow *window)
 	
 	/* Load widget state */
 	glade_window_config_load (window);
+
+#ifdef MAC_INTEGRATION
+	{
+		/* Fix up the menubar for MacOSX Quartz builds */
+		gtk_widget_hide (menubar);
+		ige_mac_menu_set_menu_bar (GTK_MENU_SHELL (menubar));
+		
+		widget = gtk_ui_manager_get_widget (window->priv->ui, "/MenuBar/FileMenu/Quit");
+		ige_mac_menu_set_quit_menu_item (GTK_MENU_ITEM (widget));
+	}
+#endif
+
+
 }
 
 static void
